# ==================================================================================
# Routine to produce Voronoi polygons. Requires modified version of voronoi.py
# ==================================================================================
  def voronoi_polygons( self ):
    import voronoi
    
    ptDict = {} 
    ptNdx = -1 
    vprovider = self.vlayer.dataProvider()
    allAttrs = vprovider.attributeIndexes()
    vprovider.select( allAttrs )
    fields = {
    0 : QgsField( "PointId", QVariant.Int ),
    1 : QgsField( "PointName", QVariant.String )}
    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
    fields, QGis.WKBPolygon, vprovider.crs() )
    inFeat = QgsFeature()
    points = []
    while vprovider.nextFeature( inFeat ):
      inGeom = QgsGeometry( inFeat.geometry() )
      point = inGeom.asPoint()
      points.append( point )
      ptNdx +=1  
      ptDict[ptNdx] = inFeat.id() 
    vprovider.rewind()
    vprovider.select( allAttrs )
    vertices,equations,edges,bounds,polygon_vertices = voronoi.computeVoronoiDiagram(points)
    feat = QgsFeature()
    nFeat = len( vertices )
    nElement = 0
    self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
    self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )

    polygon_vertices.sort()
    cur = -1
    for i,v in enumerate(polygon_vertices):
      if (not(v[0] == cur) and i > 0) or (i == len(polygon_vertices) - 1):
        polygon.append(curStart)
        feat.addAttribute( 0, QVariant( ptDict[ cur ] ) )
        geometry = QgsGeometry().fromPolygon( [ polygon ] )
        feat.setGeometry( geometry )      
        writer.addFeature( feat )
        nElement += 1
        self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ),  nElement )
      if (not(v[0] == cur)) :
        cur = v[0]
        curStart = QgsPoint(v[2],v[3])
        polygon = []
      polygon.append( QgsPoint(v[2],v[3]) )

    del writer
    return True

# ==================================================================================
# Modified triangulation that works with spatialite layers and shapefiles where a query is being
# used to subset the layer.
# ==================================================================================      
  def delaunay_triangulation( self ):
    import voronoi
    
    ptDict = {}
    ptNdx = -1
    vprovider = self.vlayer.dataProvider()
    allAttrs = vprovider.attributeIndexes()
    vprovider.select( allAttrs )
    fields = {
    0 : QgsField( "POINTA", QVariant.Double ),
    1 : QgsField( "POINTB", QVariant.Double ),
    2 : QgsField( "POINTC", QVariant.Double ) }
    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
    fields, QGis.WKBPolygon, vprovider.crs() )
    inFeat = QgsFeature()
    points = []
    while vprovider.nextFeature( inFeat ):
      inGeom = QgsGeometry( inFeat.geometry() )
      point = inGeom.asPoint()
      points.append( point )
      ptNdx +=1
      ptDict[ptNdx] = inFeat.id()
    vprovider.rewind()
    vprovider.select( allAttrs )
    triangles = voronoi.computeDelaunayTriangulation( points )
    feat = QgsFeature()
    nFeat = len( triangles )
    nElement = 0
    self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
    self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
    for triangle in triangles:
      indicies = list( triangle )
      indicies.append( indicies[ 0 ] )
      polygon = []
      step = 0
      for index in indicies:
        vprovider.featureAtId( ptDict[index], inFeat, True,  allAttrs )
        geom = QgsGeometry( inFeat.geometry() )
        point = QgsPoint( geom.asPoint() )
        polygon.append( point )
        if step <= 3: feat.addAttribute( step, QVariant( index ) )
        step += 1
      geometry = QgsGeometry().fromPolygon( [ polygon ] )
      feat.setGeometry( geometry )      
      writer.addFeature( feat )
      nElement += 1
      self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ),  nElement )
    del writer
    return True
    